@@ -18,9 +18,9 @@
 // 02111-1307, USA.
 // ========================================================================
 //
-// File          : $RCSfile$
-// Revision      : $Revision$
-// Revision date : $Date$
+// File          : $RCSfile: biff.cc,v $
+// Revision      : $Revision: 1.6 $
+// Revision date : $Date: 2004/11/26 17:42:41 $
 // Author(s)     : Nicolas Rougier
 // Short         : 
 //
@@ -32,6 +32,7 @@
 #include <fstream>
 #include <sstream>
 #include <map>
+#include <fcntl.h>
 #include "biff.h"
 #include "ui-preferences.h"
 #include "ui-properties.h"
@@ -68,14 +69,14 @@ extern "C" {
 								 GError **error)
 	{
 		BIFF(data)->xml_start_element (context, element_name, attribute_names, attribute_values, error);
-	};
+	}
 
 	void BIFF_xml_error (GMarkupParseContext *context,
 						 GError *error,
 						 gpointer data)
 	{
 		BIFF(data)->xml_error (context, error);
-	};
+	}
 }
 
 
@@ -138,8 +139,12 @@ Biff::Biff (gint ui_mode,
 
 	if (!filename.empty())
 		filename_ = filename;
-	else 
-		filename_ = std::string (g_get_home_dir ()) + std::string ("/.gnubiffrc");
+	else
+	{
+		gchar *filename=g_build_filename(g_get_home_dir (),".gnubiffrc", NULL);
+		filename_ = std::string (filename);
+		g_free(filename);
+	}
 
 	// Does configuration file exist ?
 	std::ifstream file;
@@ -224,7 +229,7 @@ Biff::lookup (void)
 }
 
 
-void
+gboolean
 Biff::lookup (Mailbox *mailbox)
 {
 	g_mutex_lock (lookup_mutex_);
@@ -233,9 +238,10 @@ Biff::lookup (Mailbox *mailbox)
 		if (mailbox_[i] == mailbox) {
 			lookup (i);
 			g_mutex_unlock (lookup_mutex_);
-			return;
+			return mailbox!=mailbox_[i];
 		}
 	g_mutex_unlock (lookup_mutex_);
+	return false;
 }
 
 void
@@ -280,11 +286,11 @@ Biff::lookup (guint index)
 		// If "properties" currently displays mailbox_[index]
 		//  we have to make it display the new mailbox
 		if (preferences_->properties()->mailbox() == mailbox_[index])
-			{
-				gdk_threads_enter();
-				preferences_->properties()->select (mailbox);
-				gdk_threads_leave();
-			}
+		{
+			gdk_threads_enter();
+			preferences_->properties()->select (mailbox);
+			gdk_threads_leave();
+		}
 		
 		// Delete old mailbox and replace it with the new one
 		Mailbox *old_mailbox = mailbox_[index];
@@ -294,6 +300,9 @@ Biff::lookup (guint index)
 	gdk_threads_enter();
 	preferences_->synchronize();
 	gdk_threads_leave();
+
+	if ((mailbox) && (check_mode_ == AUTOMATIC_CHECK))
+		mailbox->watch_on (mailbox->polltime());
 }
 
 void
@@ -306,108 +315,180 @@ Biff::lookup_thread (void)
 	g_mutex_unlock (lookup_mutex_);
 }
 
+/** 
+ * Opens the new block {\em name} of options in the configuration file.
+ *
+ * @param  name  valid utf-8 character array for the name of the block
+ */
+void
+Biff::save_newblock(const gchar *name)
+{
+	save_blocks.push_back(name);
+	const gchar *fmt="%*s<%s>\n";
+	gchar *esc=g_markup_printf_escaped(fmt,save_blocks.size()*2-2,"",name);
+	save_file << esc;
+	g_free(esc);
+}
+
+/** 
+ * Ends the last opened block of options in the configuration file.
+ */
+void
+Biff::save_endblock()
+{
+	const gchar *fmt="%*s</%s>\n";
+	gchar *esc=g_markup_printf_escaped(fmt,save_blocks.size()*2-2,"",
+									   save_blocks[save_blocks.size()-1]);
+	save_file << esc;
+	g_free(esc);
+	save_blocks.pop_back();
+}
+
+/** 
+ * Saves the string {\em value} for the option {\em name} to the
+ * configuration file.
+ *
+ * @param  name  valid utf-8 character array for the name of the option
+ * @param  value C++ string that will be saved
+ */
+void
+Biff::save_para(const gchar *name,std::string value)
+{
+	const gchar *fmt="%*s<parameter name=\"%s\"%*svalue=\"%s\"/>\n";
+	gchar *esc=g_markup_printf_escaped(fmt,save_blocks.size()*2,"",name,
+									   28-strlen(name)-save_blocks.size()*2,
+									   "",value.c_str());
+
+	save_file << esc;
+	g_free(esc);
+}
+
+/** 
+ * Saves the integer {\em value} for the option {\em name} to the
+ * configuration file.
+ *
+ * @param  name  valid utf-8 character array for the name of the option
+ * @param  value integer that will be saved
+ */
+void
+Biff::save_para(const gchar *name,gint value)
+{
+	std::stringstream value_str;
+	value_str << value;
+	save_para(name,value_str.str());
+}
+
 gboolean
 Biff::save (void)
 {
-	std::ofstream file;
+	// Note: "stringstream" and standard C file access functions are used
+	// instead of "ofstream" because there seems to be no way to set file
+	// permissions without the susceptibility to race conditions when using
+	// "ofstream". Does ofstream respect the umask command? RSo
 
-	file.open (filename_.c_str());
-	if (!file.is_open())
-		return false;
-  
-	file << "<?xml version=\"1.0\"?>" << std::endl;
-	file << "<configuration-file>"<< std::endl;
+	// XML header
+	save_blocks.clear();
+	save_file.str(std::string(""));
+	save_file << "<?xml version=\"1.0\"?>" << std::endl;
+
+	save_newblock("configuration-file");
 
 	// Mailboxes
-	for (unsigned int i=0; i< mailbox_.size(); i++) {
-		file << "  <mailbox>" << std::endl;
-		file << "    <parameter name=\"protocol\"         value=\"" << mailbox_[i]->protocol() << "\"/>" << std::endl;
-		file << "    <parameter name=\"name\"             value=\"" << mailbox_[i]->name() << "\"/>" << std::endl;
-		file << "    <parameter name=\"is_local\"         value=\"" << mailbox_[i]->is_local() << "\"/>" << std::endl;
-		file << "    <parameter name=\"location\"         value=\"" << mailbox_[i]->location() << "\"/>" << std::endl;
-		file << "    <parameter name=\"hostname\"         value=\"" << mailbox_[i]->hostname() << "\"/>" << std::endl;
-		file << "    <parameter name=\"port\"             value=\"" << mailbox_[i]->port() << "\"/>" << std::endl;
-		file << "    <parameter name=\"folder\"           value=\"" << mailbox_[i]->folder() << "\"/>" << std::endl;
-		file << "    <parameter name=\"username\"         value=\"" << mailbox_[i]->username() << "\"/>" << std::endl;
-		// pop3 and imap4 protocols requires password in clear so we have to save password
-		// in clear within configuration file. No need to say this is higly unsecure if
-		// somebody looks at the file. So we try to take some measures:
+	for (unsigned int i=0; i< mailbox_.size(); i++)
+	{
+		save_newblock("mailbox");
+		save_para("protocol",mailbox_[i]->protocol());
+		save_para("name",mailbox_[i]->name());
+		save_para("is_local",mailbox_[i]->is_local());
+		save_para("location",mailbox_[i]->location());
+		save_para("hostname",mailbox_[i]->hostname());
+		save_para("port",mailbox_[i]->port());
+		save_para("folder",mailbox_[i]->folder());
+		save_para("username",mailbox_[i]->username());
+		// pop3 and imap4 protocols requires password in clear so we have to
+		// save password in clear within configuration file. No need to say
+		// this is higly unsecure if somebody looks at the file. So we try to
+		// take some measures:
 		//   1. The file is made readable by owner only.
 		//   2. password is "crypted" so it's not directly human readable.
-		// Of course, these measures won't prevent a determined person to break in but will
-		// at least prevent "ordinary" people to be able to steal password easily.
+		// Of course, these measures won't prevent a determined person to break
+		// in but will at least prevent "ordinary" people to be able to steal
+		// password easily.
 #ifdef USE_PASSWORD
-		file << "    <parameter name=\"password\"         value=\"";
+		std::stringstream password;
 		for (guint j=0; j<mailbox_[i]->password().size(); j++)
-			file << passtable_[mailbox_[i]->password()[j]/16] << passtable_[mailbox_[i]->password()[j]%16];
-		file << "\"/>" << std::endl;
+		    password << passtable_[mailbox_[i]->password()[j]/16]
+			         << passtable_[mailbox_[i]->password()[j]%16];
+		save_para("password",password.str());
 #else
-		file << "    <parameter name=\"password\"         value=\"" << "" << "\"/>" << std::endl;
+		save_para("password",std::string(""));
 #endif
-		file << "    <parameter name=\"use_ssl\"          value=\"" << mailbox_[i]->use_ssl() << "\"/>" << std::endl;
-		file << "    <parameter name=\"certificate\"      value=\"" << mailbox_[i]->certificate() << "\"/>" << std::endl;
-		file << "    <parameter name=\"polltime\"         value=\"" << mailbox_[i]->polltime() << "\"/>" << std::endl;
-		file << "    <parameter name=\"seen\"             value=\"";
+		save_para("use_ssl",mailbox_[i]->use_ssl());
+		save_para("certificate",mailbox_[i]->certificate());
+		save_para("polltime",mailbox_[i]->polltime());
+		std::stringstream seen;
 		for (guint j=0; j<mailbox_[i]->hiddens(); j++)
-			file << mailbox_[i]->hidden(j) << " ";
-		file << "\"/>" << std::endl;
-		file << "  </mailbox>" << std::endl;
+		    seen << mailbox_[i]->hidden(j) << " ";
+		save_para("seen",seen.str());
+		save_endblock();
 	}
 
 	// General
-	file << "  <general>" << std::endl;
-	file << "    <parameter name=\"no_clear_password\" value=\"" << no_clear_password_ << "\"/>" << std::endl;
-	file << "    <parameter name=\"sound_type\"        value=\"" << sound_type_ << "\"/>" << std::endl;
-	file << "    <parameter name=\"sound_command\"     value=\"" << sound_command_ << "\"/>" << std::endl;
-	file << "    <parameter name=\"sound_file\"        value=\"" << sound_file_ << "\"/>" << std::endl;
-	file << "    <parameter name=\"sound_volume\"      value=\"" << sound_volume_ << "\"/>" << std::endl;
-	file << "    <parameter name=\"check_mode\"        value=\"" << check_mode_ << "\"/>" << std::endl;
-	file << "    <parameter name=\"max_mail\"          value=\"" << max_mail_ << "\"/>" << std::endl;
-	file << "    <parameter name=\"mail_app\"          value=\"" << mail_app_ << "\"/>" << std::endl;
-	file << "  </general>" << std::endl;
+	save_newblock("general");
+	save_para("no_clear_password",no_clear_password_);
+	save_para("sound_type",sound_type_);
+	save_para("sound_command",sound_command_);
+	save_para("sound_file",sound_file_);
+	save_para("sound_volume",sound_volume_);
+	save_para("check_mode",check_mode_);
+	save_para("max_mail",max_mail_);
+	save_para("mail_app",mail_app_);
+	save_endblock();
 
 	// Popup
-	file << "  <popup>" << std::endl;
-	file << "    <parameter name=\"popup_display\"           value=\"" << popup_display_  << "\"/>" << std::endl;
-	file << "    <parameter name=\"popup_time\"              value=\"" << popup_time_  << "\"/>" << std::endl;
-	file << "    <parameter name=\"popup_use_geometry\"      value=\"" << popup_use_geometry_ << "\"/>" << std::endl;
-	file << "    <parameter name=\"popup_geometry\"          value=\"" << popup_geometry_ << "\"/>" << std::endl;
-	file << "    <parameter name=\"popup_is_decorated\"      value=\"" << popup_is_decorated_  << "\"/>" << std::endl;
-	file << "    <parameter name=\"popup_max_line\"          value=\"" << popup_max_line_  << "\"/>" << std::endl;
-	file << "    <parameter name=\"popup_max_sender_size\"   value=\"" << popup_max_sender_size_  << "\"/>" << std::endl;
-	file << "    <parameter name=\"popup_max_subject_size\"  value=\"" << popup_max_subject_size_  << "\"/>" << std::endl;
-	file << "    <parameter name=\"popup_display_date\"      value=\"" << popup_display_date_  << "\"/>" << std::endl;
-	file << "    <parameter name=\"popup_font\"              value=\"" << popup_font_ << "\"/>" << std::endl;
-	file << "    <parameter name=\"popup_font_color\"        value=\"" << popup_font_color_  << "\"/>" << std::endl;
-	file << "  </popup>" << std::endl;
+	save_newblock("popup");
+	save_para("popup_display",popup_display_);
+	save_para("popup_time",popup_time_);
+	save_para("popup_use_geometry",popup_use_geometry_);
+	save_para("popup_geometry",popup_geometry_);
+	save_para("popup_is_decorated",popup_is_decorated_);
+	save_para("popup_max_line",popup_max_line_);
+	save_para("popup_max_sender_size",popup_max_sender_size_);
+	save_para("popup_max_subject_size",popup_max_subject_size_);
+	save_para("popup_display_date",popup_display_date_);
+	save_para("popup_font",popup_font_);
+	save_para("popup_font_color",popup_font_color_);
+	save_endblock();
 
 	// Biff
-	file << "  <biff>" << std::endl;
-	file << "    <parameter name=\"biff_newmail_image\"       value=\"" << biff_newmail_image_ << "\"/>" << std::endl;
-	file << "    <parameter name=\"biff_nomail_image\"        value=\"" << biff_nomail_image_ << "\"/>" << std::endl;
-	file << "    <parameter name=\"biff_use_newmail_image\"   value=\"" << biff_use_newmail_image_ << "\"/>" << std::endl;
-	file << "    <parameter name=\"biff_use_nomail_image\"    value=\"" << biff_use_nomail_image_ << "\"/>" << std::endl;
-	file << "    <parameter name=\"biff_use_newmail_text\"    value=\"" << biff_use_newmail_text_ << "\"/>" << std::endl;
-	file << "    <parameter name=\"biff_use_nomail_text\"     value=\"" << biff_use_nomail_text_ << "\"/>" << std::endl;
-	file << "    <parameter name=\"biff_newmail_text\"        value=\"" << biff_newmail_text_ << "\"/>" << std::endl;
-	file << "    <parameter name=\"biff_nomail_text\"         value=\"" << biff_nomail_text_ << "\"/>" << std::endl;
-	file << "    <parameter name=\"biff_use_geometry\"        value=\"" << biff_use_geometry_ << "\"/>" << std::endl;
-	file << "    <parameter name=\"biff_geometry\"            value=\"" << biff_geometry_ << "\"/>" << std::endl;
-	file << "    <parameter name=\"biff_is_decorated\"        value=\"" << biff_is_decorated_ << "\"/>" << std::endl;
-	file << "    <parameter name=\"biff_font\"                value=\"" << biff_font_ << "\"/>" << std::endl;
-	file << "    <parameter name=\"biff_font_color\"          value=\"" << biff_font_color_ << "\"/>" << std::endl;
-	file << "  </biff>" << std::endl;
+	save_newblock("biff");
+	save_para("biff_newmail_image",biff_newmail_image_);
+	save_para("biff_nomail_image",biff_nomail_image_);
+	save_para("biff_use_newmail_image",biff_use_newmail_image_);
+	save_para("biff_use_nomail_image",biff_use_nomail_image_);
+	save_para("biff_use_newmail_text",biff_use_newmail_text_);
+	save_para("biff_use_nomail_text",biff_use_nomail_text_);
+	save_para("biff_newmail_text",biff_newmail_text_);
+	save_para("biff_nomail_text",biff_nomail_text_);
+	save_para("biff_use_geometry",biff_use_geometry_);
+	save_para("biff_geometry",biff_geometry_);
+	save_para("biff_is_decorated",biff_is_decorated_);
+	save_para("biff_font",biff_font_);
+	save_para("biff_font_color",biff_font_color_);
+	save_endblock();
 
 	// End Header
-	file << "</configuration-file>"<< std::endl;
+	save_endblock();
 
-	// Close the file
-	file.close ();
+	// Write Configuration to file
+	int fd=open(filename_.c_str(),O_WRONLY|O_CREAT|O_TRUNC,S_IRUSR|S_IWUSR);
+	if (fd==-1)
+	    return false;
+	if (write(fd,save_file.str().c_str(),save_file.str().size())==-1)
+	    return false;
+	if (close(fd)==-1)
+	    return false;
 
-	// Restrict file permission
-	std::string command = std::string("chmod 600 ") + filename_;
-	system (command.c_str());
-  
 	return true;
 }
 
