@@ -18,9 +18,9 @@
 // 02111-1307, USA.
 // ========================================================================
 //
-// File          : $RCSfile$
-// Revision      : $Revision$
-// Revision date : $Date$
+// File          : $RCSfile: imap4.h,v $
+// Revision      : $Revision: 1.3 $
+// Revision date : $Date: 2004/11/17 12:39:10 $
 // Author(s)     : Nicolas Rougier
 // Short         : 
 //
@@ -38,10 +38,17 @@
 
 class Imap4 : public Mailbox {
 
+private:
+	std::string tag_;
+	guint tagcounter_;
 protected:
 	class Socket *			socket_;
 	std::vector<int>		saved_;
-
+	std::string parse_bodystructure (std::string, gint &,
+									 gboolean toplevel=true);
+	void reset_tag();
+	std::string tag();
+	gint send(std::string,gboolean debug=true);
 public:
 	/* Base */
 	Imap4 (class Biff *owner);
