@@ -18,9 +18,9 @@
 // 02111-1307, USA.
 // ========================================================================
 //
-// File          : $RCSfile$
-// Revision      : $Revision$
-// Revision date : $Date$
+// File          : $RCSfile: mailbox.h,v $
+// Revision      : $Revision: 1.4 $
+// Revision date : $Date: 2004/11/25 16:52:42 $
 // Author(s)     : Nicolas Rougier
 // Short         : 
 //
@@ -94,8 +94,11 @@ protected:
 	GMutex *					watch_mutex_;	// mutex for watch function control
 	gint 						status_;		// status of the mailbox
 	std::vector<header>			unread_;		// collected unread mail
+	std::vector<header>			new_unread_;	// collected unread mail (tmp buffer)
 	std::vector<guint>			hidden_;		// mails that won't be displayed
-	std::vector<guint>			seen_;			// mails already seen           
+	std::vector<guint>			seen_;			// mails already seen   
+	std::vector<guint>			new_seen_;		// mails already seen (tmp buffer)
+
 	static class Authentication *ui_authentication_;// ui to get username & password
 
 public:
@@ -170,4 +173,31 @@ public:
 
 };
 
+/**
+ * In some situations we need to read a certain number of lines from the
+ * network to get the line we want. Unfortunately this number may vary in
+ * reality because of the following reasons:
+ *  * There is no limit for the response
+ *  * There exist different extensions to the protocols
+ *  * Not all servers implement protocols correctly
+ *  * There is a DoS attack
+ * To prevent being DoS attacked we need to set a limit of additional lines
+ * that are being read. This is done by the following constant.
+ */
+const gint preventDoS_additionalLines_=16;
+
+/**
+ * To prevent being DoS attacked (see above): Maximum number of header lines
+ * being read (POP3).
+ */
+const gint preventDoS_headerLines_=2048;
+
+/**
+ * To prevent being DoS attacked (see above): Limit for length of a read line.
+ * SMTP: maximum line length is 1001 (see RFC 2821 4.5.3.1)
+ * IMAP: no maximum line length
+ * POP3: maximum response line length is 512 (see RFC 1939 3.)
+ */
+const gint preventDoS_lineLength_=16384;
+
 #endif
