@@ -18,9 +18,9 @@
 // 02111-1307, USA.
 // ========================================================================
 //
-// File          : $RCSfile$
-// Revision      : $Revision$
-// Revision date : $Date$
+// File          : $RCSfile: ui-popup.cc,v $
+// Revision      : $Revision: 1.3 $
+// Revision date : $Date: 2004/10/14 10:31:33 $
 // Author(s)     : Nicolas Rougier
 // Short         : 
 //
@@ -37,6 +37,7 @@
 #include "ui-popup.h"
 #include "mailbox.h"
 #include "nls.h"
+#include "support.h"
 
 
 /**
@@ -241,7 +242,7 @@ Popup::update (void)
 				// Subject
 				buffer = parse_header (biff_->mailbox(j)->unread(i).subject);
 				gchar *subject;
-				subject = g_strndup (buffer, biff_->popup_max_subject_size_);
+				subject = gb_utf8_strndup (buffer, biff_->popup_max_subject_size_);
 				g_free (buffer);
 				saved_strings.push_back (subject);
 
@@ -251,7 +252,7 @@ Popup::update (void)
 				
 				// Sender
 				gchar *buffer = parse_header (biff_->mailbox(j)->unread(i).sender);
-				gchar *sender = g_strndup (buffer, biff_->popup_max_sender_size_);
+				gchar *sender = gb_utf8_strndup (buffer, biff_->popup_max_sender_size_);
 				g_free (buffer);
 				saved_strings.push_back (sender);
 
@@ -529,7 +530,7 @@ Popup::parse_header (std::string text)
 					g_free (utf8_part);
 					utf8_text = buffer;
 				}
-				copy_part.clear();
+				copy_part.erase();
 			}
 			i+=2; 
 			if (i >= copy.size()) {
@@ -557,7 +558,7 @@ Popup::parse_header (std::string text)
 				copy_part = _("* error *");
 				break;
 			}
-			copy_part.clear();
+			copy_part.erase();
 			while ((i < copy.size()) && (copy.substr(i,2) != "?="))
 				copy_part += copy[i++];
 			if (i >= copy.size()) {
