@@ -18,9 +18,9 @@
 // 02111-1307, USA.
 // ========================================================================
 //
-// File          : $RCSfile$
-// Revision      : $Revision$
-// Revision date : $Date$
+// File          : $RCSfile: ui-preferences.cc,v $
+// Revision      : $Revision: 1.5 $
+// Revision date : $Date: 2004/10/18 09:16:14 $
 // Author(s)     : Nicolas Rougier
 // Short         : 
 //
@@ -29,6 +29,7 @@
 // -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:nil -*-
 // ========================================================================
 
+#include <cstdio>
 #include <sstream>
 #include "ui-preferences.h"
 #include "ui-properties.h"
@@ -37,6 +38,7 @@
 #include "biff.h"
 #include "mailbox.h"
 #include "nls.h"
+#include "support.h"
 
 /* "C" bindings */
 extern "C" {
@@ -108,6 +110,7 @@ Preferences::Preferences (Biff *biff) : GUI (GNUBIFF_DATADIR"/preferences.glade"
 	biff_ = biff;
 	properties_ = new Properties (this);
 	properties_->create ();
+	added_ = 0;
 }
 
 
@@ -451,22 +454,24 @@ Preferences::apply (void)
 void
 Preferences::on_add	(GtkWidget *widget)
 {
-	Mailbox *mailbox = new Mailbox(biff_);
-	biff_->add (mailbox);
-	synchronize ();
-	GtkTreeView  *view  = GTK_TREE_VIEW (get("mailboxes_treeview"));
-	GtkListStore *store = GTK_LIST_STORE (gtk_tree_view_get_model (view));
-	GtkTreeIter iter;
-	gboolean valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL(store), &iter);
-	while (valid) {
-		guint uin;
-		gtk_tree_model_get (GTK_TREE_MODEL(store), &iter, COLUMN_UIN, &uin, -1);
-		if (mailbox->uin() == uin)
-			break;
-		valid = gtk_tree_model_iter_next (GTK_TREE_MODEL (store), &iter);
+	if (added_ == 0) {
+		added_ = new Mailbox(biff_);
+		biff_->add (added_);
+		synchronize ();
+		GtkTreeView  *view  = GTK_TREE_VIEW (get("mailboxes_treeview"));
+		GtkListStore *store = GTK_LIST_STORE (gtk_tree_view_get_model (view));
+		GtkTreeIter iter;
+		gboolean valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL(store), &iter);
+		while (valid) {
+			guint uin;
+			gtk_tree_model_get (GTK_TREE_MODEL(store), &iter, COLUMN_UIN, &uin, -1);
+			if (added_->uin() == uin)
+				break;
+			valid = gtk_tree_model_iter_next (GTK_TREE_MODEL (store), &iter);
+		}
+		gtk_tree_selection_select_iter  (gtk_tree_view_get_selection (view), &iter);
+		properties_->show ();
 	}
-	gtk_tree_selection_select_iter  (gtk_tree_view_get_selection (view), &iter);
-	properties_->show ();
 }
 
 void
@@ -552,20 +557,15 @@ Preferences::on_test_sound (GtkWidget *widget)
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("system_beep_check"))))
 		gdk_beep ();    
 	else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get("sound_check")))) {
-		gchar *data = (gchar *) gtk_entry_get_text (GTK_ENTRY(get("sound_entry")));
 		std::stringstream s;
 		s << gfloat (gtk_range_get_value (GTK_RANGE (get ("volume_scale"))))/100.0;
 		std::string command = gtk_entry_get_text (GTK_ENTRY(get("command_entry")));
-		guint i;
-		while ((i = command.find ("%s")) != std::string::npos) {
-			command.erase (i, 2);
-			std::string filename = std::string("\"") + std::string(data) + std::string("\"");
-			command.insert(i, filename);
-		}
-		while ((i = command.find ("%v")) != std::string::npos) {
-			command.erase (i, 2);
-			command.insert(i, s.str());
-		}
+
+		std::vector<std::string> vec(2);
+		vec[0]=std::string(g_shell_quote(biff_->sound_file_.c_str()));
+		vec[1]=std::string(s.str());
+		command=gb_substitute(command,"sv",vec);
+
 		command += " &";
 		system (command.c_str());
 	}
@@ -582,7 +582,7 @@ Preferences::on_browse_newmail_image (GtkWidget *widget)
 {
 	GtkWidget *preview = gtk_image_new ();
 	gboolean result = browse (_("Browse for a new mail image"), "newmail_image_entry", false, preview);
-	if (result == GTK_RESPONSE_ACCEPT) {
+	if (result == 1) {
 		GtkImageAnimation *anim;
 		anim = (GtkImageAnimation *) g_object_get_data (G_OBJECT(get("newmail_image")), "_animation_");
 		anim->open ((gchar *) gtk_entry_get_text (GTK_ENTRY(get("newmail_image_entry"))));
@@ -595,7 +595,7 @@ Preferences::on_browse_nomail_image (GtkWidget *widget)
 {
 	GtkWidget *preview = gtk_image_new ();
 	gboolean result = browse (_("Browse for a new mail image"), "nomail_image_entry", false, preview);
-	if (result == GTK_RESPONSE_ACCEPT) {
+	if (result == 1) {
 		GtkImageAnimation *anim;
 		anim = (GtkImageAnimation *) g_object_get_data (G_OBJECT(get("nomail_image")), "_animation_");
 		anim->open ((gchar *) gtk_entry_get_text (GTK_ENTRY(get("nomail_image_entry"))));
