@@ -18,9 +18,9 @@
 // 02111-1307, USA.
 // ========================================================================
 //
-// File          : $RCSfile$
-// Revision      : $Revision$
-// Revision date : $Date$
+// File          : $RCSfile: ui-properties.cc,v $
+// Revision      : $Revision: 1.1 $
+// Revision date : $Date: 2004/10/06 13:21:57 $
 // Author(s)     : Nicolas Rougier
 // Short         : 
 //
@@ -124,6 +124,7 @@ void
 Properties::on_ok (GtkWidget *widget)
 {
 	on_apply (widget);
+	preferences_->added (0);
 	hide ();
 }
 
@@ -154,6 +155,13 @@ void
 Properties::on_cancel (GtkWidget *widget)
 {
 	hide ();
+	Preferences *prefs = preferences_;
+	if (prefs->added()) {
+		prefs->biff()->remove (prefs->added()->uin());
+		prefs->added(0);
+		prefs->synchronize ();		
+	}
+
 }
 
 void
@@ -172,7 +180,7 @@ gboolean
 Properties::on_destroy (GtkWidget *widget,
 						GdkEvent *event)
 {
-	hide();
+	on_cancel(0);
 	return TRUE;
 }
 
@@ -180,6 +188,6 @@ gboolean
 Properties::on_delete	(GtkWidget *widget,
 						 GdkEvent *event)
 {
-	hide ();
+	on_cancel(0);
 	return TRUE;
 }
